@@ -32,6 +32,10 @@
 #include <string>
 #include <csignal>
 
+#include <re2/re2.h>
+
+#include "stdlib.h"
+
 //#include "astream.h"
 #include "atomic_unicode_histogram.h"
 #include "sbuf.h"
@@ -746,26 +750,77 @@ TEST_CASE("pos0_t", "[feature_recorder]") {
  * regex_vector.h & regex_vector.cpp
  */
 #include "regex_vector.h"
+char disable[64];
 TEST_CASE("test regex_vector", "[regex]") {
     REQUIRE(regex_vector::has_metachars("this[1234]foo") == true);
     REQUIRE(regex_vector::has_metachars("this(1234)foo") == true);
     REQUIRE(regex_vector::has_metachars("this[1234].*foo") == true);
     REQUIRE(regex_vector::has_metachars("this1234foo") == false);
 
-    regex_vector rv;
-    rv.push_back("this.*");
-    rv.push_back("check[1-9]");
-    rv.push_back("thing");
+    for(int pass=0;pass<3;pass++){
+        if (pass==0) {
+            strncpy(disable,"RE_ENGINE=RE2",sizeof(disable));
+            putenv(disable);
+        }
+        if (pass==1) {
+            strncpy(disable,"RE_ENGINE=PCRE",sizeof(disable));
+            putenv(disable);
+        }
+        if (pass==2) {
+            strncpy(disable,"RE_ENGINE=STD::REGEX",sizeof(disable));
+            putenv(disable);
+        }
 
-    REQUIRE(rv.size() == 3);
+        regex_vector rv;
+        std::cout << "Pass " << pass << " testing regex engine: " <<  rv.regex_engine() << std::endl;
 
-    std::string found;
-    REQUIRE(rv.search_all("hello1", &found) == false);
-    REQUIRE(rv.search_all("check1", &found) == true);
-    REQUIRE(found == "check1");
+        REQUIRE(rv.size() == 0);
+        rv.push_back("this.*");
+        REQUIRE(rv.size() == 1);
+        rv.clear();			// test this functionality
+        REQUIRE(rv.size() == 0);
 
-    REQUIRE(rv.search_all("before check2 after", &found) == true);
-    REQUIRE(found == "check2");
+        rv.push_back("this.*");
+        rv.push_back("check[1-9]");
+        rv.push_back("thing");
+
+        REQUIRE(rv.size() == 3);
+
+        std::string found;
+        REQUIRE(rv.search_all("hello1", &found) == false);
+        REQUIRE(rv.search_all("check1", &found) == true);
+        REQUIRE(found == "check1");
+
+        REQUIRE(rv.search_all("before check2 after", &found) == true);
+        REQUIRE(found == "check2");
+
+        rv.clear();
+        rv.push_back("[a-z]*@company.com");
+
+        /* This tests to make sure searching for a potentially open-ended string doesn't
+         * take a very long time. We do the search in a 30MiB array...
+         */
+        const size_t mbytes = 1;
+        const size_t bytes = 1024*1024*mbytes;
+        std::string bigstring = std::string(bytes,'a')
+            + " user@company.com "
+            + std::string(1024*1024*2,'b');
+        found="";
+        size_t offset = 0;
+        size_t len = 0;
+        aftimer t;
+        t.start();
+        alarm(6000);
+        REQUIRE(rv.search_all(bigstring, &found, &offset, &len) == true);
+        alarm(0);
+        t.stop();
+        std::cout << "time=" << t.elapsed_seconds() << "\n";
+        REQUIRE(found == "user@company.com");
+        REQUIRE(offset == bytes+1);
+        REQUIRE(len == 16 );
+    }
+    strncpy(disable,"RE_ENGINE=RE2",sizeof(disable));
+    putenv(disable);
 }
 
 /****************************************************************
@@ -1225,8 +1280,34 @@ TEST_CASE("word_and_context_list", "[feature_recorder]") {
     REQUIRE(word_and_context_list::rstrcmp("aaaa1", "bbbb0") < 0);
     REQUIRE(word_and_context_list::rstrcmp("aaaa1", "aaaa1") == 0);
     REQUIRE(word_and_context_list::rstrcmp("bbbb0", "aaaa1") > 0);
+
+    auto tmpdir = NamedTemporaryDirectory();
+    std::filesystem::path words_txt = tmpdir / "words.txt";
+    std::filesystem::path bogus_txt = tmpdir / "bogus.txt";
+    std::ofstream os;
+    os.open(words_txt);
+    REQUIRE(os.is_open());
+    os << "# This is a comment\n";
+
+    // Three features with no context
+    os << "word1\n";
+    os << "word2\n";
+    os << "word3\n";
+
+    // And a simple regular expression
+    os << "letters?\n";
+
+    os.close();
+    word_and_context_list wcl;
+    REQUIRE(wcl.readfile(bogus_txt) == -1);
+    REQUIRE(wcl.readfile(words_txt) == 0);
+    REQUIRE(wcl.size() == 4);
+
+    std::filesystem::remove(words_txt);
+    std::filesystem::remove_all(tmpdir);
 }
 
+
 /****************************************************************
  * unicode_escape.h
  * unicode_escape.cpp
