@@ -56,13 +56,16 @@ int _CRT_fmode = _O_BINARY;
 [[noreturn]] void debug_help()
 {
     puts( "#define DEBUG_PEDANTIC    0x0001	// check values more rigorously" );
-    puts( "#define DEBUG_PRINT_STEPS 0x0002      // prints as each scanner is started" );
+    puts( "#define DEBUG_PRINT_STEPS 0x0002     // prints as each scanner is started" );
     puts( "#define DEBUG_SCANNER     0x0004	// dump all feature writes to stderr" );
-    puts( "#define DEBUG_NO_SCANNERS 0x0008      // do not run the scanners " );
+    puts( "#define DEBUG_NO_SCANNERS 0x0008     // do not run the scanners " );
     puts( "#define DEBUG_DUMP_DATA   0x0010	// dump data as it is seen " );
     puts( "#define DEBUG_INFO        0x0040	// print extra info" );
     puts( "#define DEBUG_EXIT_EARLY  1000	// just print the size of the volume and exis " );
     puts( "#define DEBUG_ALLOCATE_512MiB 1002	// Allocate 512MiB, but don't set any flags " );
+    puts( "// any debug can also be enabled by setting the corresponding environment variables");
+    puts( "// e.g. DEBUG_PRINT_STEPS=1 ./bulk_extractor ...");
+
     exit( 1);
 }
 
@@ -140,6 +143,7 @@ void bulk_extractor_set_debug()
     sbuf_t::debug_leak            = getenv_debug("DEBUG_SBUF_LEAK");
 }
 
+bool RUNNING_UNDER_CATCH = false;
 int bulk_extractor_main( std::ostream &cout, std::ostream &cerr, int argc,char * const *argv)
 {
     bulk_extractor_set_debug();
@@ -150,7 +154,7 @@ int bulk_extractor_main( std::ostream &cout, std::ostream &cerr, int argc,char *
     const auto original_argv = argv;
 
     // Only print these warnings if not running under catch
-    if (getenv("RUNNING_UNDER_CATCH")==nullptr){
+    if (RUNNING_UNDER_CATCH == false ){
 #ifdef HAVE_ADDRESS_SANITIZER
         cerr << "*** compiled with address sanitizer" << std::endl;
 #endif
@@ -271,6 +275,12 @@ int bulk_extractor_main( std::ostream &cout, std::ostream &cerr, int argc,char *
         sc.banner_file = result["banner_file"].as<std::string>();
     } catch ( cxxopts::option_has_no_value_exception &e ) { }
 
+    try {
+        for ( const auto &name : result["disable"].as<std::vector<std::string>>() ) {
+            sc.push_scanner_command( name, scanner_config::scanner_command::DISABLE);
+        }
+    } catch ( cxxopts::option_has_no_value_exception &e ) { }
+
     if ( result.count( "enable_exclusive" )) {
         sc.push_scanner_command( scanner_config::scanner_command::ALL_SCANNERS, scanner_config::scanner_command::DISABLE);
         sc.push_scanner_command( result["enable_exclusive"].as<std::string>(), scanner_config::scanner_command::ENABLE);
@@ -282,22 +292,25 @@ int bulk_extractor_main( std::ostream &cout, std::ostream &cerr, int argc,char *
         }
     } catch ( cxxopts::option_has_no_value_exception &e ) { }
 
-
-    try {
-        for ( const auto &name : result["disable"].as<std::vector<std::string>>() ) {
-            sc.push_scanner_command( name, scanner_config::scanner_command::DISABLE);
-        }
-    } catch ( cxxopts::option_has_no_value_exception &e ) { }
-
     cfg.opt_pagesize   = scaled_stoi64( result["pagesize"].as<std::string>());
     cfg.opt_marginsize = scaled_stoi64( result["marginsize"].as<std::string>());
 
+    /*** SET THREADING OPTIONS ***/
+    if ( result.count("threads")>0 && result.count("no_threads") >0) {
+        throw std::runtime_error("--threads and --no_threads conflict");
+    }
+
     try {
         cfg.num_threads    = result["threads"].as<int>();
     }  catch ( cxxopts::option_has_no_value_exception &e ) {
         cfg.num_threads = 0;
     }
 
+    if ( result.count("no_threads")) {
+        cfg.num_threads = 0;
+    }
+    /***/
+
     sc.max_depth             = result["max_depth"].as<int>();
     cfg.max_bad_alloc_errors = result["max_bad_alloc_errors"].as<int>();
 
@@ -375,11 +388,14 @@ int bulk_extractor_main( std::ostream &cout, std::ostream &cerr, int argc,char *
     //sc.get_global_config( "write_feature_sqlite3",&opt_write_sqlite3,"Write feature files to report.sqlite3" );
     sc.get_global_config( "report_read_errors",&cfg.opt_report_read_errors,"Report read errors" );
 
-    /* If we are getting help or info scanners, make a fake scanner set with new output directory */
+    /* If we are getting help or info scanners, make a fake scanner set with new output directory,
+     * then apply the scanner commands so we can get the feature recorders created...
+     */
     if ( result.count( "help" ) || result.count( "info_scanners" )) {
         struct feature_recorder_set::flags_t f;
         scanner_set ss( sc, f, nullptr);     // make a scanner_set but with no XML writer. We will create it below
         ss.add_scanners( scanners_builtin);
+        ss.apply_scanner_commands();
 
         if ( result.count( "help" )) {     // -h
             cout << options.help() << std::endl;
@@ -413,6 +429,7 @@ int bulk_extractor_main( std::ostream &cout, std::ostream &cerr, int argc,char *
     } catch ( cxxopts::option_has_no_value_exception &e ) { }
 
     try {
+        /* For each 'find_file' option, add its path to the scanner config */
         for ( const auto &it : result["find_file"].as<std::vector<std::string>>() ) {
             sc.add_find_path( it );
         }
